from dataclasses import dataclass
from typing import List, Optional

from govuk_frontend_django.components import base as govuk_frontend_base
from govuk_frontend_django.components import (
    error_message as govuk_frontend_error_message,
)
from govuk_frontend_django.components import fieldset as govuk_frontend_fieldset
from govuk_frontend_django.components import hint as govuk_frontend_hint
from govuk_frontend_django.components import label as govuk_frontend_label
from govuk_frontend_django.components import tag as govuk_frontend_tag


@dataclass(kw_only=True)
class HeaderNavigation:
    text: Optional[str] = None
    html: Optional[str] = None
    href: Optional[str] = None
    active: Optional[bool] = None
    attributes: Optional[govuk_frontend_base.Attributes] = None


@dataclass(kw_only=True)
class GovUKHeader(govuk_frontend_base.GovUKComponent):
    """GOV.UK Header

    See: https://design-system.service.gov.uk/components/header/
    """

    homepageUrl: Optional[str] = None
    productName: Optional[str] = None
    serviceName: Optional[str] = None
    serviceUrl: Optional[str] = None
    navigation: Optional[List[HeaderNavigation]] = None
    navigationClasses: Optional[str] = None
    navigationLabel: Optional[str] = None
    menuButtonLabel: Optional[str] = None
    menuButtonText: Optional[str] = None
    containerClasses: Optional[str] = None
    useTudorCrown: Optional[bool] = None

    _jinja2_template = "govuk_frontend_jinja/components/header/macro.html"
    _macro_name = "govukHeader"


COMPONENT = GovUKHeader
